#-*- coding:utf8 -*-

import numpy as np
# import matplotlib.pyplot as plt
import copy
import re

# import error

def readData(filename):
    with open(filename) as fd:
        datax = []
        datay = []
        for ln in fd:
            out = re.findall('[+-]?\d+\.?\d*(?:e[+-]?\d+)?',ln)
            if len(out)<=1:
                continue
            assert len(out)>1
            datax.append(float(out[0]))
            datay.append(float(out[1]))

    return datax,datay
    

def getWeight(i,r,data,beta1,beta2,sigma = 10e-6):
    #return 1.0
    return 1.0/(sigma+(data[i+r]-(beta1+ r*beta2))**2)**0.5


def tau(w,n):
    w = np.array(w)
    s1 = w.sum()
    s2 = w.dot(np.arange(-n+1,n+1))
    s3 = w.dot([i*i for i in xrange(-n+1,n+1)])
    return s1,s2,s3
    
def solveBeta(i,data,n):
    count = 0
    data_part = data[i-n+1:i+n+1]
    beta2 = sum([(6*r-3)*data[i+r] for r in xrange(-n+1,n+1)])/(4.0*n**3-n)
    beta1 = data_part.sum()/(2.0*n)-beta2/2.0
    w = np.array([getWeight(i,r, data, beta1, beta2) for r in xrange(-n+1,n+1)])
    betaold1 = -100
    betaold2 = -100
    while count < 500:
        tau1,tau2,tau3 = tau(w,n)
        count += 1
        tmp = data_part*w
        beta2 = tmp.dot([1.0*(r*tau1-tau2)/(tau1*tau3-tau2*tau2)
                         for r in xrange(-n+1,n+1)])
        beta1 = 1.0*tmp.sum()/(tau1)-beta2*(1.0*tau2/tau1)
        compare = max(abs(betaold1-beta1),abs(betaold2-beta2))
        betaold1 = beta1
        betaold2 = beta2

        w = np.array(
            [getWeight(i,r, data, beta1, beta2) for r in xrange(-n+1,n+1)])
        if count >= 1 and compare < 10e-6:
            break

    return beta1,beta2,w

    
def calcf1(w,tau1,tau2,tau3,i,data,n):
    tmp =[(1-(tau2-0.25*tau1)*((1.0*jj*tau1-tau2)/(tau1*tau3 - tau2*tau2)))
           *w[jj+n-1]*data[i+jj] for jj in xrange(-n+1,n+1)]
    return 1.0*sum(tmp)/tau1

def calcf2(w,tau1,tau2,tau3,i,data,n):
    tmp =[(1-(tau2-0.75*tau1)*((1.0*jj*tau1-tau2)/(tau1*tau3 - tau2*tau2)))
           *w[jj+n-1]*data[i+jj] for jj in xrange(-n+1,n+1)]
    return 1.0*sum(tmp)/tau1

def getAll(data,n):
    out = []
    for ii in xrange(n-1,len(data)-n):
        beta1,beta2,w = solveBeta(ii,data,n)
        w = np.array(w)
        tau1,tau2,tau3 = tau(w,n)
        
        out.append(calcf1(w,tau1,tau2,tau3,ii,data,n))
        out.append(calcf2(w,tau1,tau2,tau3,ii,data,n))
                   
    return np.array(out)

def color(col):
    return plt.cm.jet(col/0.2)

def plot(datax,datay,err):
    xy = np.array([datax,datay]).T
    fig, ax = plt.subplots()
    for start, stop ,col in zip(xy[:-1], xy[1:],err[1:]):
        x, y = zip(start, stop)
        ax.plot(x, y, color= color(col))

    return fig, ax


if __name__=='__main__':
####    n = 3     # Here n is for scheme that is S1,S2,S3..
####
######    filename = raw_input("File name:")
####    filename = "data5"
####    
####    datax,datay = readData(filename)
####    datax = np.array(datax)
####    datay = np.array(datay)
####
####    dataxinput = datax.copy()
####    datayinput = datay.copy()
####
####    #plt.plot(datax,datay,'o',label=u"Data")
####    #plt.show()
####
####    for x in xrange(2):
####        datax = getAll(datax,n)
####        datay = getAll(datay,n)
######        np.savez(filename+'_'+str(2*n)+'_'+str(xx+1),x = datax,y = datay,z = dataz)
####
####    err = abs(error.error(datax,datay))
####    errmax = err.max()
####
####    xy = np.array([datax,datay]).T
####    fig, ax = plt.subplots()
####    for start, stop ,col in zip(xy[:-1], xy[1:],err[1:]):
####        x, y = zip(start, stop)
####        ax.plot(x, y, color= plt.cm.jet(col/errmax))
####    #plt.show()
####        
####    #plt.plot(dataxinput[n+1:-n-1],datayinput[n+1:-n-1],'ko',label=u"Input")
####    ax.plot(datax,error.function(datax),':',label=u"$f_1$")
####    #ax.imshow(np.array([datax,datay]).T,shape=datax/datax.max(),label=u"Result",aspect='auto',cmap = plt.cm.jet)
####    #plt.colorbar(tt) 
####    #plt.legend()
####    #fig.colorbar(tt)
####    plt.show()
####
######    cur = []
######    for i in xrange(1,len(datax)-1):
######        ddf = (datay[i+1]+datay[i-1]-2*datay[i])/(datax[i+1]-datax[i])**2
######        df = (datay[i+1]-datay[i-1])/(datax[i+1]-datax[i-1])
######        cur.append(ddf/(1+df**2)**1.5)
######
######    plt.close()
######    plt.plot(datax[1:-1],cur)
######    plt.show()



    n = 3
    times = 5
    pretimes = 5

    filename = "data5"
    datax,datay = readData(filename+ '.txt')
    dataxinput = np.array(datax)
    datayinput = np.array(datay)
    if pretimes :
        tmp = np.load(filename+"_"+str(2*n)+"_" +str(pretimes)+'.npz')
        datax = tmp['x']
        datay = tmp['y']
    else :            
        datax = dataxinput.copy()
        datay = datayinput.copy()
    


    for xx in xrange(pretimes,times):
        print 'Start',xx+1,'times...'
        datax = getAll(datax,n)
        datay = getAll(datay,n)
        np.savez(filename+'_'+str(2*n)+'_'+str(xx+1),x = datax,y = datay)

    print 'Painting...'

    err = abs(error.error(datax,datay))
    print err.max()

    fig,ax = plot(datax,datay,err)
    ax.plot(datax,error.function(datax),':',label=u"$f_1$")
    ax.plot(datax[:1],datay[:1],color = color(0),label = u"Result")
    ax.plot(dataxinput[n+1:-n-1],datayinput[n+1:-n-1],'ko',markersize = 3,label=u"Input")
    plt.legend()
    plt.savefig(filename+"_"+str(2*n)+"_" +str(pretimes)+'.pdf')
    plt.show()






















