#-*- coding:utf8 -*-
from __future__ import division
import copy
import re

import numpy as np
# import matplotlib.pyplot as plt

# import error

def readData(filename):
    with open(filename) as fd:
        datax = []
        datay = []
        for ln in fd:
            out = re.findall('[+-]?\d+\.?\d*(?:e[+-]?\d+)?',ln)
            if len(out)<=1:
                continue
            assert len(out)>1
            datax.append(float(out[0]))
            datay.append(float(out[1]))

    return datax,datay
    

def getWeight(i,r,data,beta1,beta2,sigma = 10e-6):
    return 1.0/(sigma+(data[i+r]-(beta1+ r*beta2))**2)**0.5
    #return 1.0


def tau(w,n):
    w = np.array(w)
    s1 = w.sum()
    s2 = w.dot(np.arange(-n,n+1))
    s3 = w.dot([i*i for i in xrange(-n,n+1)])
    return s1,s2,s3
    
def solveBeta(i,data,n):
    count = 0
    data_part = data[i-n:i+n+1]
    beta2 = sum([3*r*data[i+r] for r in xrange(-n,n+1)])/(n*(n+1)*(2*n+1))
    beta1 = data_part.sum()/(2.0*n+1)
    w = np.array([getWeight(i,r, data, beta1, beta2) for r in xrange(-n,n+1)])
    betaold1 = -100
    betaold2 = -100
    while count < 500:
        tau1,tau2,tau3 = tau(w,n)
        count += 1
        tmp = data_part*w
        beta2 = tmp.dot([1.0*(r*tau1-tau2)/(tau1*tau3-tau2*tau2)
                         for r in xrange(-n,n+1)])
        beta1 = 1.0*tmp.sum()/(tau1)-beta2*(1.0*tau2/tau1)
        compare = max(abs(betaold1-beta1),abs(betaold2-beta2))
        betaold1 = beta1
        betaold2 = beta2

        w = np.array(
            [getWeight(i,r, data, beta1, beta2) for r in xrange(-n,n+1)])
        if count >= 1 and compare < 10e-6:
            break

    return beta1,beta2,w

    
def calcf1(w,tau1,tau2,tau3,i,data,n):
    tmp =[(1-(tau2-0.25*tau1)*((1.0*jj*tau1-tau2)/(tau1*tau3 - tau2*tau2)))
           *w[jj+n]*data[i+jj] for jj in xrange(-n,n+1)]
    return 1.0*sum(tmp)/tau1

def calcf2(w,tau1,tau2,tau3,i,data,n):
    tmp =[(1-(tau2+0.25*tau1)*((1.0*jj*tau1-tau2)/(tau1*tau3 - tau2*tau2)))
           *w[jj+n]*data[i+jj] for jj in xrange(-n,n+1)]
    return 1.0*sum(tmp)/tau1

def getAll(data,n):
    out = []
    for ii in xrange(n,len(data)-n):
        beta1,beta2,w = solveBeta(ii,data,n)
        w = np.array(w)
        tau1,tau2,tau3 = tau(w,n)
        
        out.append(calcf2(w,tau1,tau2,tau3,ii,data,n))
        out.append(calcf1(w,tau1,tau2,tau3,ii,data,n))
                   
    return np.array(out)


if __name__=='__main__':
    n = 3     # Here n is for scheme that is S1,S2,S3..

    filename = raw_input("File name:")
    
    datax,datay = readData(filename)
    datax = np.array(datax)
    datay = np.array(datay)

    dataxinput = datax.copy()
    datayinput = datay.copy()

##    plt.plot(datax,datay,'o',label=u"Data")
##    plt.show()

    for x in xrange(4):
        datax = getAll(datax,n)
        datay = getAll(datay,n)

    print error.error(datax,datay)
    plt.plot(dataxinput[n+2:-n-2],datayinput[n+2:-n-2],'o',linewidth=1,label=u"Input")
    plt.plot(datax,error.function(datax),'-',label=u"$f_1$")
    plt.plot(datax,datay,linewidth=5,label=u"Result")
##    The follow two to change the font size of tha x-axis,y-axis
##    you can change the size as you want. You can also copy this
##    two lines to the 2n curve scheme.
    plt.xticks(fontsize=12)
    plt.yticks(fontsize=12)
    
    plt.legend()
    plt.show()



























