#-*- coding:utf8 -*-

from __future__ import division
import numpy as np
import copy
import re
import Weight2dOdd as weight2

def readDataOfPara(filename):
    with open(filename) as fd:
        ln = fd.readline()
        m,n = map(int,re.findall('\d+',ln))

        datax = []
        datay = []
        dataz = []
        for ln in fd:
            out = re.findall('[+-]?\d+\.?\d*(?:e[+-]?\d+)?',ln)
            if len(out)!=3:
                continue
            datax.append(float(out[0]))
            datay.append(float(out[1]))
            dataz.append(float(out[2]))

    return (np.array(datax).reshape(m,n),
            np.array(datay).reshape(m,n),
            np.array(dataz).reshape(m,n)
            )

def readData(filename):
    with open(filename) as fd:
        ln = fd.readline()
        out = re.findall('\d+',ln)
        if len(out) != 2:
            print 'First line in the file is wrong!'
            raise
        mm = int(out[0])
        nn = int(out[1])

        ln = fd.readline()
        out = re.findall('[+-]?\d+\.?\d*(?:e[+-]?\d+)?',ln)
        assert mm == len(out)
        datax = map(float,out)
        datax = np.array(datax)
        datax.shape = mm,1
        ln = fd.readline()
        out = re.findall('[+-]?\d+\.?\d*(?:e[+-]?\d+)?',ln)
        assert nn == len(out)
        datay = map(float,out)
        datay = np.array(datay)
        datay.shape = 1,nn
        
        dataz = []
        count = 0
        for ln in fd:
            out = re.findall('[+-]?\d+\.?\d*(?:e[+-]?\d+)?',ln)
            if not len(out):
                continue
            elif len(out) != nn:
                print 'Error data format!',out
                raise
                
            dataz.append(map(float,out))
            count += 1
        assert count == mm

    return datax,datay,np.array(dataz)
    
def getWeight(i,j,r,s,data,beta1,beta2,beta3,sigma = 10e-6):
    return 1.0/(sigma+(data[i+r][j+s]-(beta1 + beta2*r+beta3*s))**2)**0.5

def getTao(w,n):
    s1 = w.sum()
    s2 = sum(
        [r*w[r+n][s+n] for r in xrange(-n,n+1) for s in xrange(-n,n+1)])
    s3 = sum(
        [s*w[r+n][s+n] for r in xrange(-n,n+1) for s in xrange(-n,n+1)])
    s4 = sum(
        [r*r*w[r+n][s+n] for r in xrange(-n,n+1) for s in xrange(-n,n+1)])
    s5 = sum(
        [r*s*w[r+n][s+n] for r in xrange(-n,n+1) for s in xrange(-n,n+1)])
    s6 = sum(
        [s*s*w[r+n][s+n] for r in xrange(-n,n+1) for s in xrange(-n,n+1)])
    return s1,s2,s3,s4,s5,s6

def getLambda(t1,t2,t3,t4,t5,t6):
    l1 = t4*t6 - t5*t5
    l2 = t2*t6 - t3*t5
    l3 = t2*t5 - t3*t4
    l4 = t1*t6 - t3*t3
    l5 = t1*t5 - t2*t3
    l6 = t1*t4 - t2*t2
    return l1,l2,l3,l4,l5,l6

def getK(t1,t2,t3,t4,t5,t6):
    k = t1*t4*t6 - t1*t5*t5 - t2*t2*t6 + 2*t2*t3*t5 - t3*t3*t4
    return k

def getX(i,j,w,f,n):
    x1 = sum([w[r+n][s+n]*f[i+r][j+s]
         for r in xrange(-n,n+1) for s in xrange(-n,n+1)])    
    x2 = sum([r*w[r+n][s+n]*f[i+r][j+s]
         for r in xrange(-n,n+1) for s in xrange(-n,n+1)])
    
    x3 = sum([s*w[r+n][s+n]*f[i+r][j+s]
         for r in xrange(-n,n+1) for s in xrange(-n,n+1)])
    return x1,x2,x3

    
def solveBeta(i,j,data,n):
    count = 0
    beta1 = sum([data[i+r][j+s]
         for r in xrange(-n,n+1) for s in xrange(-n,n+1)]
         )/((2*n+1)*(2*n+1))
    beta2 = sum([(r)*data[i+r][j+s]
         for r in xrange(-n,n+1) for s in xrange(-n,n+1)]
         )*(3)/(n*(n+1)*(2*n+1)*(2*n+1))
    beta3 = sum([(s)*data[i+r][j+s]
         for r in xrange(-n,n+1) for s in xrange(-n,n+1)]
         )*(3)/(n*(n+1)*(2*n+1)*(2*n+1))

    w = np.array([getWeight(i,j,r,s,data,beta1,beta2,beta3)
                  for r in xrange(-n,n+1) for s in xrange(-n,n+1)])
    w.shape = 2*n+1,2*n+1
    betaold1 = -100
    betaold2 = -100
    betaold3 = -100
    while count < 50:
        tao1,tao2,tao3,tao4,tao5,tao6 = getTao(w,n)
        x1,x2,x3 = getX(i,j,w,data,n)
        k = getK(tao1,tao2,tao3,tao4,tao5,tao6)
        l1,l2,l3,l4,l5,l6 = getLambda(tao1,tao2,tao3,tao4,tao5,tao6)
        count += 1
        beta1 = 1.0*(l1*x1 - l2*x2 + l3*x3)/k
        beta2 = 1.0*(-l2*x1 + l4*x2 - l5*x3)/k
        beta3 = 1.0*(l3*x1 - l5*x2 + l6*x3)/k
        compare = max(
            abs(betaold1-beta1),abs(betaold2-beta2),abs(betaold3-beta3))
        betaold1 = beta1
        betaold2 = beta2
        betaold3 = beta3
        w = np.array([getWeight(i,j,r,s,data,beta1,beta2,beta3)
                  for r in xrange(-n,n+1) for s in xrange(-n,n+1)])
        w.shape = 2*n+1,2*n+1
        if count > 1 and compare < 10e-6:
            break
    return beta1,beta2,beta3,w

def getPhi(r,s,l1,l2,l3):
    return l1-l2*r+l3*s

def getAll(data,n):
    mm,nn = data.shape
    out = np.array([[0.0]*2*(nn-2*n)]*2*(mm-2*n), dtype = np.float64)
    for i in xrange(n,mm-n):
        for j in xrange(n,nn-n):
            beta1,beta2,beta3,w = solveBeta(i,j,data,n)

            out[2*(i-n)][2*(j-n)]=beta1-beta2*0.25-beta3*0.25
            out[2*(i-n)+1][2*(j-n)]=beta1+beta2*0.25-beta3*0.25
            out[2*(i-n)][2*(j-n)+1]=beta1-beta2*0.25+beta3*0.25
            out[2*(i-n)+1][2*(j-n)+1]=beta1+beta2*0.25+beta3*0.25

    return np.array(out)


if __name__=='__main__':
    n = 3
    times = 5
    pretimes = 5

    filename = "D_data3d1"
    if pretimes == 0 :
        datax,datay,dataz = readData(filename+ '.txt')
        mm,nn = dataz.shape
        datax.shape = mm
        datay.shape = nn
    else :
        tmp = np.load(filename+"_"+str(2*n+1)+"_" +str(pretimes)+'.npz')
        datax = tmp['x']
        datay = tmp['y']
        dataz = tmp['z']
    
    a = dataz.copy()

    for xx in xrange(pretimes,times):
        print 'Start',xx+1,'times...'
        datax = weight2.getAll(datax,n)
        datay = weight2.getAll(datay,n)
        dataz = getAll(dataz,n)
        np.savez(filename+'_'+str(2*n+1)+'_'+str(xx+1),x = datax,y = datay,z = dataz)

    datax.shape = datax.shape+(1,)
    datay.shape = (1,)+datay.shape

    print 'Painting...'

    from errorsurf import error
    err = abs(error(datax,datay,dataz))
    print "max error:",err.max()

    fi = mlab.figure(bgcolor = (1.0,1.0,1.0),size = (1000,1000))
    ss = mlab.mesh(datax+0*datay,0*datax +datay,dataz,
                   vmax = 0.29,vmin=0,
                   scalars = err,colormap = "jet")
    mlab.savefig(filename +"_"+str(2*n+1) +"_"+str(times)+ ".png",figure = fi, size =(3000,3000))
    mlab.show()

