#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import argparse

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='l1-Regression based subdivision schemes for noisy data.')
    parser.add_argument('-t',dest='times', type=int,default=5,
                        help="The times of iterate. The default value is 5.")
    parser.add_argument('-n', dest='num', type = int, default=6,
                        help = "Number of subdivision scheme. The default value is 6.")
    parser.add_argument('-d', dest='dim', type = int, default=2, choices = [2,3],
                        help = "The dim of subdivision scheme. The default value is 2.")
    parser.add_argument('-p', dest='parameter',action='store_false',
                        help = "Show the 3d data is parametric surface or not. The default is not using the parametric surface.")
    parser.add_argument('--version', action='version', version='%(prog)s 0.9')
    parser.add_argument('input', help="The name of data file, without the extension.For example, if the data file is 'data.txt', Then the parameter here should be 'data'.")

    args = parser.parse_args()

    dim = args.dim
    ##  gn is the number of scheme.
    gn = args.num
    n = gn//2
    ##  you can change the times of iterate here.
    times = args.times
    ##  Pretimes is what times you had iterated, it will load the result of data
    ##  had already saved. For example, you have run the code with 4 iterate times,
    ##  and now you want calculate the 5 times, then you can let the pretimes = 4,
    ##  the code will start at 5th time. Don't calculate the first 4 times.
    ##  You can just generate the figure let pretimes = times.
    pretimes = 0
    ##  Here is the name of the data, it's used to load and save data in a list of files.
    ##  If you change the name, and want to use the saved data, you should rename the file
    ##  of the .npz file also.
    filename = args.input
    is_not_para = args.parameter

    if dim == 2:
        if gn%2:
            from Weight2dOdd import *
        else :
            from Weight2dEven import *

        datax,datay = readData(filename+ '.txt')
        dataxinput = np.array(datax)
        datayinput = np.array(datay)
        if pretimes :
            tmp = np.load(filename+"_"+str(gn)+"_" +str(pretimes)+'.npz')
            datax = tmp['x']
            datay = tmp['y']
        else :
            datax = dataxinput.copy()
            datay = datayinput.copy()

        for xx in xrange(pretimes,times):
            print 'Start',xx+1,'times...'
            datax = getAll(datax,n)
            datay = getAll(datay,n)
            np.savez(filename+'_'+str(gn)+'_'+str(xx+1),x = datax,y = datay)

        print 'Painting...'

        import matplotlib.pyplot as plt
        fig, ax = plt.subplots()
        ax.plot(datax,datay,label = u"Result")
        ax.plot(dataxinput[n+1:-n-1],datayinput[n+1:-n-1],'ko',markersize = 3,label=u"Input")
        plt.legend()

        ##  You can change the style of figure here, and change the font size.
        plt.savefig(filename+"_"+str(gn)+"_" +str(times)+'.pdf')
        plt.show()

    elif dim == 3:
        if gn%2:
            from Weight3dOdd import *
        else :
            from Weight3dEven import *

        if is_not_para :
            if pretimes == 0 :
                datax,datay,dataz = readData(filename+ '.txt')
                mm,nn = dataz.shape
                datax.shape = mm
                datay.shape = nn
            else :
                tmp = np.load(filename+"_"+str(gn)+"_" +str(pretimes)+'.npz')
                datax = tmp['x']
                datay = tmp['y']
                dataz = tmp['z']

            a = dataz.copy()

            for xx in xrange(pretimes,times):
                print 'Start',xx+1,'times...'
                datax = weight2.getAll(datax,n)
                datay = weight2.getAll(datay,n)
                dataz = getAll(dataz,n)
                np.savez(filename+'_'+str(gn)+'_'+str(xx+1),x = datax,y = datay,z = dataz)

            datax.shape = datax.shape+(1,)
            datay.shape = (1,)+datay.shape

        else:
            if pretimes == 0 :
                datax,datay,dataz = readDataOfPara(filename + '.txt')
            else :
                tmp = np.load(filename+"_"+str(gn)+"_" +str(pretimes)+'.npz')
                datax = tmp['x']
                datay = tmp['y']
                dataz = tmp['z']

            a = datax.copy()
            b = datay.copy()
            c = dataz.copy()

            for x in xrange(pretimes,times):
                print 'Start',x+1,'times...'
                datax = getAll(datax,n)
                datay = getAll(datay,n)
                dataz = getAll(dataz,n)

                np.savez(filename+'_'+str(gn)+'_'+str(x+1),x = datax,y = datay,z = dataz)

        from mayavi import mlab
        print 'Painting...'

        fi = mlab.figure(bgcolor = (1.0,1.0,1.0),size = (1000,1000))
        if is_not_para:
            ss = mlab.mesh(datax+0*datay,0*datax +datay,dataz)
        else :
            ss = mlab.mesh(datax,datay,dataz)
        mlab.savefig(filename +"_"+str(gn) +"_"+str(times)+ ".png",figure = fi)
        mlab.show()

